package net.sf.sql2java.generator.writer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringReader;

/**
 * Write the files with regards to usercode.
 *
 * If the file contains certain code tags, this won't be overwritten.
 *
 * @author daang
 */
public class CheckedCodeWriter extends CodeWriter {

// -- Writing -----------------------------------------------------------------
    
    public void write(String path, String content) throws Exception {
        // -- we need to add the target directory to the actual path
        //path = appendTargetDirectory(path);
        
        // -- create a file for the path
        File file = new File(path);
        
        // -- inject usercode
        content = injectUserCode(file, content);
        
        // -- wrap the file into a writer so we can put data in it
        PrintWriter writer = new PrintWriter(new OutputStreamWriter(new FileOutputStream(file)));

        // -- write our content to the file
        writer.write(content);
        writer.flush();
        writer.close();
    }
    
    public String injectUserCode(File file, String content) {
        // -- create the result object
        String result = content;
        
        try {
            // -- first check the filetype
            //String type = getExtension(file.getName());
        
            // -- create the merger
            UsercodeMerger merger = new UsercodeMerger();
        
            // -- if the file does not exists, create the directories
            if (! file.exists())
                (new File(file.getParent())).mkdirs();
            
            // -- and start merging
            if (merger != null)
                result = merger.mergeUsercode(
                        new StringReader(content), 
                        (file.exists()) ? new FileReader(file) : null
                );
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        // -- return the result object
        return result;
    }
    
// -- Helpers -----------------------------------------------------------------
    
//    private String getExtension(String filename) {
//        String[] parts = filename.split("\\.");
//        
//        if (parts.length > 0)
//            return parts[parts.length - 1];
//        else
//            return "";
//    }
//    
//    private String appendTargetDirectory(String path) {
//        // TODO : FIX THIS
//        /*if (! getTarget().getPath().endsWith("/"))
//            targetDirectory += "/";
//        
//        return targetDirectory + path;*/
//        return null;
//    }
}
