package net.sf.sql2java.packages;

import java.util.HashMap;
import java.util.Map;

import net.sf.sql2java.common.configuration.Configurer;
import net.sf.sql2java.common.store.PackStore;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.log4j.Logger;

public class PackageConfigurer extends Configurer<PackStore> {
	private static final Logger LOG = Logger.getLogger(PackageConfigurer.class);

// -- Configuration -----------------------------------------------------------	

	@Override
	public void configure(PackStore subject) throws ConfigurationException {
		LOG.info("Configuring packStore.");
		
		// -- check if the configuration is valid
		if (getConfiguration() == null)
			throw new ConfigurationException("No configuration has been set");
		
		// -- get the packs
		Map<String, PackageReference> packs = getPacks();
		subject.setPacks(packs);
	}

// -- Helpers -----------------------------------------------------------------
	
	public Map<String, PackageReference> getPacks() {
		// -- create the result
		Map<String, PackageReference> result = new HashMap<String, PackageReference>();
		
		int size = getConfiguration().getList("packages.pack[@name]").size();
		for (int i = 0; i < size; i++) {
			// -- create the package reference
			PackageReference reference = createPack(i);
			
			// -- put the reference into the result map
			result.put(reference.getName(), reference);
			
			LOG.info("found new pack '" + reference.getName() + "' at " + reference.getResource());
		}
		
		// -- return the result
		return result;
	}
	
	public PackageReference createPack(int i) {
		// -- create the result
		PackageReference result = new PackageReference();
		
		// -- create the id
		String id = "packages.pack(" + i + ")";
		
		result.setName(getConfiguration().getString(id + "[@name]"));
		result.setResource(getConfiguration().getString(id + "[@resource]"));
		
		// -- return the result
		return result;
	}
	
}
