/*
 * PackageReader.java
 * 
 * Created on Jul 11, 2007, 12:51:08 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package net.sf.sql2java.packages;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.sql2java.generator.scripting.Script;
import net.sf.sql2java.generator.template.Template;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;

/**
 *
 * @author dge2
 */
public abstract class PackageReader {
    
    public abstract TemplatePack readPack(String packPath) throws IOException, ConfigurationException;
    
    protected TemplatePack readDescriptor(File file) throws ConfigurationException {
    	// -- create the descriptor configuration
    	XMLConfiguration descriptor = new XMLConfiguration(file);

        // -- read the descriptor and create the result object
        TemplatePack result = getPack(descriptor);
        
        // -- return the result
        return result;
    }
    
    public TemplatePack getPack(Configuration descriptor) {
    	// -- create the result object
    	TemplatePack result = new TemplatePack();
    	
    	// -- set the common data
    	result.setDescription(descriptor.getString("description"));
    	result.setVersion(descriptor.getString("version"));
    	result.setName(descriptor.getString("name"));
    	
    	// -- set the scripts
    	result.setScripts(getScripts(descriptor, result));
    	
    	// -- set the templates
    	result.setTemplates(getTemplates(descriptor, result));
    	
    	// -- return the result
    	return result;
    }
    
    @SuppressWarnings("unchecked")
	public Map<String, Script> getScripts(Configuration descriptor, TemplatePack templatePack) {
    	// -- create the result
    	Map<String, Script> result = new HashMap<String, Script>();
    	
    	// -- get all the script names. it indicates how much scripts 
    	// -- there are
    	List scripts = descriptor.getList("scripts.script[@name]");
    	
    	// -- iterate the templates
    	for (int i = 0; i < scripts.size(); i++) {
    		Script script = getScript(descriptor, templatePack, i);
    		
    		// -- add the template if it's valid
    		if (script != null)
    			result.put(script.getName(), script);
    	}
    	
    	// -- return the result
    	return result;
    }
    
    @SuppressWarnings("unchecked")
	protected Map<String, Template> getTemplates(Configuration descriptor, TemplatePack templatePack) {
    	// -- create the result
    	Map<String, Template> result = new HashMap<String, Template>();
    	
    	// -- get all the template names. it indicates howmuch templates 
    	// -- there are
    	List templates = descriptor.getList("templates.template[@name]");
    	
    	// -- iterate the templates
    	for (int i = 0; i < templates.size(); i++) {
    		Template template = getTemplate(descriptor, templatePack, i);
    		
    		// -- add the template if it's valid
    		if (template != null)
    			result.put(template.getName(), template);
    	}
    	
    	// -- return the result
    	return result;
    }
    
    protected Template getTemplate(Configuration descriptor, TemplatePack templatePack, int sequence) {
    	// -- create the result
    	Template result = new Template();
    	result.setTemplatePack(templatePack);

    	// -- get the templatePath
    	String path = getTemplatePath(sequence);
    	
    	// -- get the template values
    	result.setName(descriptor.getString(path + "[@name]"));
    	result.setResource(descriptor.getString(path + "[@resource]"));
    	result.setScope(descriptor.getString(path + "[@scope]").toLowerCase());
    	
    	result.setPkg(descriptor.getString(path + ".target[@package]"));
    	result.setFilename(descriptor.getString(path + ".target[@filename]"));
    	result.setExtension(descriptor.getString(path + ".target[@extension]"));
    	
    	// -- return the result
    	return result;
    }
    
    protected Script getScript(Configuration descriptor, TemplatePack templatePack, int sequence) {
    	// -- create the result
    	Script result = new Script();
    	result.setTemplatePack(templatePack);
    	
    	// -- get the scriptpath
    	String path = getScriptPath(sequence);
    	
    	// -- get the script values
    	result.setName(descriptor.getString(path + "[@name]"));
    	result.setResource(descriptor.getString(path + "[@resource]"));
    	result.setEngineType(descriptor.getString(path + "[@engine]"));
    	
    	// -- return the result
    	return result;
    }
    
    protected String getTemplatePath(int sequence) {
    	StringBuffer result = new StringBuffer();
    	
    	result.append("templates.template(");
    	result.append(sequence);
    	result.append(")");
    	
    	return result.toString();
    }
    
    protected String getScriptPath(int sequence) {
    	StringBuffer result = new StringBuffer();
    	
    	result.append("scripts.script(");
    	result.append(sequence);
    	result.append(")");
    	
    	return result.toString();
    }
    
    protected String readFile(InputStream stream) throws IOException {
        StringBuffer contents = new StringBuffer();
        BufferedReader input = null;
        
        try {
            input = new BufferedReader(new InputStreamReader(stream));
            String line = null;
            
            while (( line = input.readLine()) != null){
                contents.append(line);
                contents.append(System.getProperty("line.separator"));
            }
        } finally {
            if (input!= null)
                input.close();
        }
        
        // -- return the result
        return contents.toString();
    }
}
